@@ -87,9 +87,35 @@ struct _ChamplainPathLayerPrivate
   gdouble *dash;
   guint num_dashes;
   
-  ClutterContent *canvas;
   cairo_surface_t *surface;
+
+  /* In order to correctly render paths in the horizontal wrap,
+   * the path_actor (a map-wide actor) contains two children that
+   * split the visible paths.
+   *
+   * The right_actor renders paths visible on the original map layer.
+   * (from viewport's x coordinate to the rightmost point on the map)
+
+   * The left_actor renders paths visible on the first cloned map layer.
+   * (a fixed size, from the leftmost point on the map)
+   *
+   * If horizontal wrap is disabled, the left_actor won't render
+   * anything.
+   */
   ClutterActor *path_actor;
+
+  ClutterActor *right_actor;
+  ClutterActor *left_actor;
+
+  ClutterContent *right_canvas;
+  ClutterContent *left_canvas;
+
+  cairo_surface_t *right_surface;
+  cairo_surface_t *left_surface;
+
+  gboolean right_surface_updated;
+  gboolean left_surface_updated;
+
   GList *nodes;
   gboolean redraw_scheduled;
 };
@@ -225,13 +251,17 @@ champlain_path_layer_dispose (GObject *object)
   if (priv->view != NULL)
     set_view (CHAMPLAIN_LAYER (self), NULL);
 
-  if (priv->canvas)
+  if (priv->right_canvas)
     {
-      g_object_unref (priv->canvas);
-      priv->canvas = NULL;
+      g_object_unref (priv->right_canvas);
+      g_object_unref (priv->left_canvas);
+      priv->right_canvas = NULL;
+      priv->left_canvas = NULL;
     }
 
   g_clear_pointer (&priv->surface, cairo_surface_destroy);
+  g_clear_pointer (&priv->right_surface, cairo_surface_destroy);
+  g_clear_pointer (&priv->left_surface, cairo_surface_destroy);
 
   G_OBJECT_CLASS (champlain_path_layer_parent_class)->dispose (object);
 }
@@ -379,6 +409,18 @@ champlain_path_layer_class_init (ChamplainPathLayerClass *klass)
       "surface");
 }
 
+static void
+initialize_child_actor (ChamplainPathLayer *self,
+    ClutterActor *child_actor,
+    ClutterContent *canvas)
+{
+  ChamplainPathLayerPrivate *priv = self->priv;;
+
+  clutter_actor_set_content (child_actor, canvas);
+  g_signal_connect (canvas, "draw", G_CALLBACK (redraw_path), self);
+  clutter_actor_set_size (child_actor, 255, 255);
+  clutter_actor_add_child (priv->path_actor, child_actor);
+}
 
 static void
 champlain_path_layer_init (ChamplainPathLayer *self)
@@ -401,14 +443,28 @@ champlain_path_layer_init (ChamplainPathLayer *self)
   priv->fill_color = clutter_color_copy (&DEFAULT_FILL_COLOR);
   priv->stroke_color = clutter_color_copy (&DEFAULT_STROKE_COLOR);
 
-  priv->canvas = clutter_canvas_new ();
-  clutter_canvas_set_size (CLUTTER_CANVAS (priv->canvas), 255, 255);
-  g_signal_connect (priv->canvas, "draw", G_CALLBACK (redraw_path), self);
-
   priv->path_actor = clutter_actor_new ();
-  clutter_actor_set_size (priv->path_actor, 255, 255);
-  clutter_actor_set_content (priv->path_actor, priv->canvas);
   clutter_actor_add_child (CLUTTER_ACTOR (self), priv->path_actor);
+  clutter_actor_set_size (priv->path_actor, 255, 255);
+
+  priv->right_actor = clutter_actor_new ();
+  priv->left_actor = clutter_actor_new ();
+
+  priv->right_canvas = clutter_canvas_new ();
+  priv->left_canvas = clutter_canvas_new ();
+
+  priv->surface = NULL;
+  priv->right_surface = NULL;
+  priv->left_surface = NULL;
+
+  priv->right_surface_updated = FALSE;
+  priv->left_surface_updated = FALSE;
+
+  clutter_canvas_set_size (CLUTTER_CANVAS (priv->right_canvas), 255, 255);
+  clutter_canvas_set_size (CLUTTER_CANVAS (priv->left_canvas), 0, 0);
+
+  initialize_child_actor (self, priv->right_actor, priv->right_canvas);
+  initialize_child_actor (self, priv->left_actor, priv->left_canvas);
 }
 
 
@@ -468,20 +524,89 @@ champlain_path_layer_new ()
 }
 
 
+static void
+get_map_size (ChamplainView *view, gint *width, gint *height)
+{
+  gint size, rows, cols;
+  ChamplainMapSource *map_source = champlain_view_get_map_source (view);
+  gint zoom_level = champlain_view_get_zoom_level (view);
+  size = champlain_map_source_get_tile_size (map_source);
+  rows = champlain_map_source_get_row_count (map_source,
+                                                zoom_level);
+  cols = champlain_map_source_get_column_count (map_source,
+                                                zoom_level);
+  if (width)
+    *width = size * rows;
+
+  if (height)
+    *height = size * cols;
+}
+
+
 static gboolean
 invalidate_canvas (ChamplainPathLayer *layer)
 {
   ChamplainPathLayerPrivate *priv = layer->priv;
-  gfloat width, height;
-  
-  width = 256;
-  height = 256;
+  gfloat view_width, view_height;
+  gint map_width, map_height;
+  gint viewport_x, viewport_y;
+  gint anchor_x, anchor_y;
+  gfloat right_actor_width, right_actor_height;
+  gfloat left_actor_width, left_actor_height;
+
+  right_actor_width = 256;
+  right_actor_height = 256;
+  left_actor_width = 0;
+  left_actor_height = 0;
+  map_width = 256;
+  map_height = 256;
+
   if (priv->view != NULL)
-    clutter_actor_get_size (CLUTTER_ACTOR (priv->view), &width, &height);
+    {
+      get_map_size (priv->view, &map_width, &map_height);
+      clutter_actor_get_size (CLUTTER_ACTOR (priv->view), &view_width, &view_height);
+      champlain_view_get_viewport_origin (priv->view, &viewport_x, &viewport_y);
+      champlain_view_get_viewport_anchor (priv->view, &anchor_x, &anchor_y);
+
+      /* For efficiency in terms of clipping, the path actors must have a minimal size.
+       * The right_actor renders the paths on the visible side of the original map layer
+       * (from viewport offset to end of the map).
+       * The left_actor renders the paths on the visible side of the first cloned map layer
+       * (from the leftmost point on the map, clamped by the viewport width).
+       */
+      right_actor_width = MIN (map_width - (viewport_x + anchor_x), (gint)view_width);
+      right_actor_height = MIN (map_height - (viewport_y + anchor_y), (gint)view_height);
+      left_actor_width = MIN (view_width - right_actor_width, map_width - right_actor_width);
+      left_actor_height = right_actor_height;
+
+      /* Ensure sizes are positive  */
+      right_actor_width = MAX (0, right_actor_width);
+      right_actor_height = MAX (0, right_actor_height);
+      left_actor_width = MAX (0, left_actor_width);
+      left_actor_height = MAX (0, left_actor_height);
+    }
+
+  clutter_actor_set_size (priv->path_actor, map_width, map_height);
+
+  clutter_actor_set_size (priv->right_actor, right_actor_width, right_actor_height);
+  clutter_canvas_set_size (CLUTTER_CANVAS (priv->right_canvas), right_actor_width, right_actor_height);
+  priv->right_surface_updated = FALSE;
+  clutter_content_invalidate (priv->right_canvas);
+
+  /* Since the left actor only renders paths visible on the clone, it should be hidden
+   * when no clone is visible.
+   */
+  if (left_actor_width != 0)
+    {
+      clutter_actor_set_size (priv->left_actor, left_actor_width, left_actor_height);
+      clutter_canvas_set_size (CLUTTER_CANVAS (priv->left_canvas), left_actor_width, left_actor_height);
+      priv->left_surface_updated = FALSE;
+      clutter_content_invalidate (priv->left_canvas);
+      clutter_actor_show (priv->left_actor);
+    }
+  else
+    clutter_actor_hide (priv->left_actor);
 
-  clutter_canvas_set_size (CLUTTER_CANVAS (priv->canvas), width, height);
-  clutter_actor_set_size (priv->path_actor, width, height);
-  clutter_content_invalidate (priv->canvas);
   priv->redraw_scheduled = FALSE;
 
   return FALSE;
@@ -664,6 +789,60 @@ relocate_cb (G_GNUC_UNUSED GObject *gobject,
   schedule_redraw (layer);
 }
 
+static void
+update_surface (ChamplainPathLayer *layer,
+    ClutterCanvas *canvas,
+    cairo_surface_t *surface)
+{
+  ChamplainPathLayerPrivate *priv = layer->priv;
+
+  if (canvas == CLUTTER_CANVAS (priv->right_canvas))
+    {
+      cairo_surface_destroy (priv->right_surface);
+      priv->right_surface = cairo_surface_reference (surface);
+      priv->right_surface_updated = TRUE;
+    }
+  else if (canvas == CLUTTER_CANVAS (priv->left_canvas))
+    {
+      cairo_surface_destroy (priv->left_surface);
+      priv->left_surface = cairo_surface_reference (surface);
+      priv->left_surface_updated = TRUE;
+    }
+
+ /* Updating the exportable surface. Path layer has two surfaces (one for each canvas)
+  * which have to be merged into a single new one.
+  */
+  if (priv->left_surface_updated && priv->right_surface_updated)
+    {
+      gfloat view_width, view_height;
+      gint map_width, viewport_x, anchor_x;
+      cairo_surface_t *new_surface;
+      cairo_t *cr;
+
+      get_map_size (priv->view, &map_width, NULL);
+      clutter_actor_get_size (CLUTTER_ACTOR (priv->view), &view_width, &view_height);
+      champlain_view_get_viewport_origin (priv->view, &viewport_x, NULL);
+      champlain_view_get_viewport_anchor (priv->view, &anchor_x, NULL);
+
+      new_surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, view_width, view_height);
+      cr = cairo_create (new_surface);
+
+      cairo_set_source_surface (cr,
+                                priv->right_surface,
+                                0, 0);
+      cairo_paint (cr);
+
+      cairo_set_source_surface (cr,
+                                priv->left_surface,
+                                map_width - viewport_x - anchor_x, 0);
+      cairo_paint (cr);
+
+      set_surface (CHAMPLAIN_EXPORTABLE (layer), new_surface);
+
+      cairo_surface_destroy (new_surface);
+      cairo_destroy (cr);
+    }
+}
 
 static gboolean
 redraw_path (ClutterCanvas *canvas,
@@ -675,7 +854,8 @@ redraw_path (ClutterCanvas *canvas,
   ChamplainPathLayerPrivate *priv = layer->priv;
   GList *elem;
   ChamplainView *view = priv->view;
-  gint x, y;
+  gint  viewport_x, viewport_y;
+  gint anchor_x, anchor_y;
   
   /* layer not yet added to the view */
   if (view == NULL)
@@ -684,8 +864,13 @@ redraw_path (ClutterCanvas *canvas,
   if (!priv->visible || width == 0.0 || height == 0.0)
     return FALSE;
 
-  champlain_view_get_viewport_origin (priv->view, &x, &y);
-  clutter_actor_set_position (priv->path_actor, x, y);
+  champlain_view_get_viewport_origin (priv->view, &viewport_x, &viewport_y);
+  champlain_view_get_viewport_anchor (priv->view, &anchor_x, &anchor_y);
+
+  if (canvas == CLUTTER_CANVAS (priv->right_canvas))
+    clutter_actor_set_position (priv->right_actor, viewport_x, viewport_y);
+  else
+    clutter_actor_set_position (priv->left_actor, -anchor_x, viewport_y);
 
   /* Clear the drawing area */
   cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
@@ -702,7 +887,10 @@ redraw_path (ClutterCanvas *canvas,
       x = champlain_view_longitude_to_x (view, champlain_location_get_longitude (location));
       y = champlain_view_latitude_to_y (view, champlain_location_get_latitude (location));
 
-      cairo_line_to (cr, x, y);
+      if (canvas == CLUTTER_CANVAS (priv->right_canvas))
+        cairo_line_to (cr, x, y);
+      else
+        cairo_line_to (cr, x + (viewport_x + anchor_x), y);
     }
 
   if (priv->closed_path)
@@ -729,7 +917,7 @@ redraw_path (ClutterCanvas *canvas,
   if (priv->stroke)
     cairo_stroke (cr);
 
-  set_surface (CHAMPLAIN_EXPORTABLE (layer), cairo_get_target (cr));
+  update_surface (layer, canvas, cairo_get_target (cr));
 
   return FALSE;
 }
